Implements Commanded.EventStore.TypeProvider behavior. Using macros to generate the behavior.
Summary
Functions
Imports all the types from another module defined by Trogon.Commanded.TypeProvider.
Registers a mapping from a Protobuf message module using its full_name/0 function as the type.
Registers a mapping from an name string to an Elixir Module that defines a struct.
Functions
Imports all the types from another module defined by Trogon.Commanded.TypeProvider.
Example
defmodule UserTypeProvider do
use Trogon.Commanded.TypeProvider
# ...
end
defmodule MyAppTypeProvider do
use Trogon.Commanded.TypeProvider
import_type_provider UserTypeProvider
end
Registers a mapping from a Protobuf message module using its full_name/0 function as the type.
Example
defmodule MyTypeProvider do
use Trogon.Commanded.TypeProvider
register_protobuf_message MyApp.Proto.AccountCreated
end
Registers a mapping from an name string to an Elixir Module that defines a struct.
Example
defmodule MyTypeProvider do
use Trogon.Commanded.TypeProvider,
prefix: "accounts." # optional, adds the prefix to the type name
register_type "account_created", AccountCreated
end